package de.szut.lf8_starter.project.action.employee;

import de.szut.lf8_starter.employee.EmployeeService;
import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectService;
import de.szut.lf8_starter.project.dto.project.GetProjectDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

import java.util.Optional;

@RestController
public class AddEmployeeToProjectAction {
    private final ProjectService projectService;
    private final EmployeeService employeeService;

    public AddEmployeeToProjectAction(ProjectService projectService, EmployeeService employeeService) {
        this.projectService = projectService;
        this.employeeService = employeeService;
    }

    @Operation(summary = "Add an employee to a project")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "Employee added to project"),
            @ApiResponse(responseCode = "404", description = "Project or employee not found", content = @Content),
            @ApiResponse(responseCode = "409", description = "Project dates conflict", content = @Content)
    })
    @PostMapping("/projects/{projectId}/employees/{employeeId}")
    public ResponseEntity<Object> create(
            @PathVariable Long projectId,
            @PathVariable Long employeeId,
            @RequestHeader("Authorization") String accessToken
    ) {
        Optional<ProjectEntity> optionalProject = this.projectService.findById(projectId);

        if (optionalProject.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        ProjectEntity projectEntity = optionalProject.get();

        if (!this.employeeService.employeeExists(accessToken, employeeId)) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        for (GetProjectDto getProjectDto : this.employeeService.getProjects(employeeId)) {
            if (this.projectService.isOverlapping(getProjectDto, projectEntity)) {
                return new ResponseEntity<>("Project dates conflict with an existing project for Employee with ID: " + employeeId, HttpStatus.CONFLICT);
            }
        }

        projectEntity.getEmployees().add(employeeId);
        this.projectService.update(projectEntity);

        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }
}
